import * as knex from 'knex';
import Model from './model';
import { Pool } from 'generic-pool';
import { Database } from 'sql.js';
import * as types from './types';
export declare class Trilogy {
    isNative: boolean;
    knex: knex;
    options: types.TrilogyOptions;
    pool?: Pool<Database>;
    verbose?: (query: string) => any;
    Model: typeof Model;
    private _definitions;
    constructor(path: string, options?: types.TrilogyOptions);
    static connect(path: string, options?: types.TrilogyOptions): Trilogy;
    readonly models: string[];
    model<D extends types.ReturnDict = types.LooseObject>(name: string, schema: types.SchemaRaw<D>, options?: types.ModelOptions): Promise<Model<D>>;
    getModel<D extends types.ReturnDict = types.LooseObject>(name: string): Model<D> | never;
    hasModel(name: string): Promise<boolean>;
    dropModel(name: string): Promise<boolean>;
    raw(query: knex.QueryBuilder | knex.Raw, needResponse?: boolean): Promise<any>;
    close(): PromiseLike<void>;
    create<T = types.LooseObject>(table: string, object: types.LooseObject, options?: types.LooseObject): Promise<T>;
    find<T = types.LooseObject>(location: string, criteria?: types.Criteria, options?: types.FindOptions): Promise<T[]>;
    findOne<T = types.LooseObject>(location: string, criteria?: types.Criteria, options?: types.FindOptions): Promise<T>;
    findOrCreate<T = types.LooseObject>(table: string, criteria: types.Criteria, creation?: types.LooseObject, options?: types.FindOptions): Promise<T>;
    update(table: string, criteria: types.Criteria, data: types.LooseObject, options?: types.UpdateOptions): Promise<number>;
    updateOrCreate(table: string, criteria: types.Criteria, data: types.LooseObject, options?: types.CreateOptions & types.UpdateOptions): Promise<number>;
    get<T = types.ReturnType>(location: string, criteria: types.Criteria, defaultValue?: T): Promise<T>;
    set<T>(location: string, criteria: types.Criteria, value: T): Promise<number>;
    getRaw<T>(location: string, criteria: types.Criteria, defaultValue: T): Promise<T>;
    getRaw(location: string, criteria: types.Criteria): Promise<types.ReturnType>;
    setRaw<T>(location: string, criteria: types.Criteria, value: T): Promise<number>;
    incr(location: string, criteria: types.Criteria, amount?: number): Promise<number>;
    decr(location: string, criteria: types.Criteria, amount?: number, allowNegative?: boolean): Promise<number>;
    remove(location: string, criteria: types.Criteria): Promise<number>;
    clear(location: string): Promise<number>;
    count(location?: string, criteria?: types.Criteria, options?: types.AggregateOptions): Promise<number>;
    min(location: string, criteria: types.Criteria, options?: types.AggregateOptions): Promise<number | undefined>;
    max(location: string, criteria: types.Criteria, options?: types.AggregateOptions): Promise<number | undefined>;
}
export { default as Model, ModelParams } from './model';
export * from './types';
export * from './plugins';
export declare const connect: (path: string, options?: {
    client?: "sqlite3" | "sql.js" | undefined;
    connection?: {
        filename?: string | undefined;
    } | undefined;
    dir?: string | undefined;
    verbose?: Function | undefined;
} | undefined) => Trilogy;
