import { Trilogy } from '.';
import { Cast } from './schema-helpers';
import * as types from './types';
export declare const MODEL_FLAG: unique symbol;
export declare type ModelParams<D extends types.ReturnDict = types.LooseObject> = [Trilogy, string, types.SchemaRaw<D>, types.ModelOptions?];
export default class Model<D extends types.ReturnDict = types.LooseObject> {
    ctx: Trilogy;
    name: string;
    options: types.ModelOptions;
    cast: Cast<D>;
    schema: types.Schema<D>;
    constructor(ctx: Trilogy, name: string, schema: types.SchemaRaw<D>, options: types.ModelOptions);
    static readonly [MODEL_FLAG]: boolean;
    static [Symbol.hasInstance](object: any): boolean;
    create(object: D, options?: types.LooseObject): Promise<D | undefined>;
    find(criteria?: types.Criteria<D>, options?: types.FindOptions): Promise<D[]>;
    findIn(column: keyof D, criteria?: types.Criteria<D>, options?: types.FindOptions): Promise<types.ValueOf<D>[]>;
    findOne(criteria?: types.Criteria<D>, options?: types.FindOptions): Promise<D | undefined>;
    findOneIn<K extends keyof D = keyof D, V = D[K]>(column: K, criteria?: types.Criteria<D>, options?: types.FindOptions): Promise<V | undefined>;
    findOrCreate(criteria: types.CriteriaObj<D>, creation?: Partial<D>, options?: types.FindOptions): Promise<D | undefined>;
    update(criteria?: types.Criteria<D>, data?: Partial<D>, options?: types.UpdateOptions): Promise<number>;
    updateOrCreate(criteria: types.CriteriaObj<D>, data: Partial<D>, options?: types.UpdateOptions & types.CreateOptions): Promise<number>;
    get<K extends keyof D = keyof D, V extends D[K] = D[K]>(column: K, criteria?: types.Criteria<D>, defaultValue?: V): Promise<D[K] | undefined>;
    set<K extends keyof D = keyof D, V extends D[K] = D[K]>(column: K, criteria?: types.Criteria<D>, value?: V): Promise<number>;
    getRaw<K extends keyof D = keyof D, V extends D[K] = D[K]>(column: K, criteria: types.Criteria<D>, defaultValue?: V): Promise<D[K] | undefined>;
    setRaw<K extends keyof D = keyof D, V extends D[K] = D[K]>(column: K, criteria?: types.Criteria<D>, value?: V): Promise<number>;
    incr(column: keyof D, criteria?: types.Criteria<D>, amount?: number): Promise<number>;
    decr(column: keyof D, criteria?: types.Criteria<D>, amount?: number, allowNegative?: boolean): Promise<number>;
    remove(criteria: types.Criteria<D>): Promise<number>;
    clear(): Promise<number>;
    count(criteria?: types.Criteria<D>, options?: types.AggregateOptions): Promise<number>;
    countIn(column: keyof D, criteria?: types.Criteria<D>, options?: types.AggregateOptions): Promise<number>;
    min(column: keyof D, criteria?: types.Criteria<D>, options?: types.AggregateOptions): Promise<number | undefined>;
    max(column: keyof D, criteria?: types.Criteria<D>, options?: types.AggregateOptions): Promise<number | undefined>;
}
