"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const generic_pool_1 = require("generic-pool");
const util_1 = require("./util");
function readDatabase(instance) {
    const SQL = require('sql.js');
    const name = instance.options.connection.filename;
    if (name === ':memory:') {
        return new SQL.Database();
    }
    let client;
    try {
        util_1.makeDirPath(path_1.dirname(name));
        const file = fs_1.readFileSync(name);
        client = new SQL.Database(file);
    }
    catch (e) {
        if (e.code === 'ENOENT') {
            client = new SQL.Database();
            writeDatabase(instance, client);
        }
        else {
            throw e;
        }
    }
    return client;
}
exports.readDatabase = readDatabase;
function writeDatabase(instance, db) {
    const name = instance.options.connection.filename;
    if (name === ':memory:')
        return;
    util_1.makeDirPath(path_1.dirname(name));
    fs_1.writeFileSync(name, db.export(), { mode: parseInt('0777', 8) });
}
exports.writeDatabase = writeDatabase;
function pureConnect(instance) {
    return generic_pool_1.createPool({
        create() {
            return Promise.resolve(readDatabase(instance));
        },
        destroy(client) {
            client.close();
            return Promise.resolve(undefined);
        }
    }, { min: 1, max: 1 });
}
exports.pureConnect = pureConnect;
