import * as t from 'io-ts';
import { Raw, SchemaBuilder, QueryBuilder } from 'knex';
import { Either } from 'fp-ts/lib/Either';
export declare const raise: (validation: Either<t.ValidationError[], any>) => void;
export declare function validate<L>(value: L, type: t.Type<L>, defaultValue?: L): L;
export declare function withDefault<T extends t.Mixed>(type: T, defaultValue: (() => t.TypeOf<T>) | t.TypeOf<T>): t.Type<t.TypeOf<T>, t.TypeOf<T>>;
export interface Thenable<T> {
    then<U>(onFulfilled?: (value: T) => U | Thenable<U>, onRejected?: (error: any) => U | Thenable<U>): Thenable<U>;
}
export declare type Query = Raw | QueryBuilder | SchemaBuilder | Thenable<any>;
export declare type DistinctArrayTuple<T, V = any> = T extends [string, string, V] ? [string, string, V] : T extends [string, V] ? [string, V] : T extends V[] ? V[] : V;
export declare type StringKeys<D = LooseObject> = Extract<keyof D, string>;
export declare type LooseObject = Record<string, any>;
export declare type ValueOf<D> = D[keyof D];
export declare type Criteria2<D = LooseObject> = [StringKeys<D>, D[StringKeys<D>]];
export declare type Criteria3<D = LooseObject> = [StringKeys<D>, string, D[StringKeys<D>]];
export declare type CriteriaObj<D = LooseObject> = Partial<D>;
export declare type CriteriaBase<D = LooseObject> = Criteria2<DistinctArrayTuple<D>> | Criteria3<DistinctArrayTuple<D>> | Partial<D>;
export declare type CriteriaList<D = LooseObject> = CriteriaBase<DistinctArrayTuple<D>>[];
export declare type Criteria<D = LooseObject> = CriteriaBase<D> | CriteriaList<D>;
export declare const Index: t.UnionC<[t.StringC, t.ArrayC<t.UnionC<[t.StringC, t.ArrayC<t.StringC>]>>, t.RecordC<t.StringC, t.UnionC<[t.StringC, t.ArrayC<t.StringC>]>>]>;
export declare const GroupClause: t.UnionC<[t.StringC, t.ArrayC<t.StringC>]>;
export declare const OrderClause: t.UnionC<[t.StringC, t.TupleC<[t.StringC, t.StringC]>]>;
export declare const TrilogyOptions: t.PartialC<{
    client: t.Type<"sqlite3" | "sql.js", "sqlite3" | "sql.js", unknown>;
    connection: t.Type<{
        filename?: string | undefined;
    }, {
        filename?: string | undefined;
    }, unknown>;
    dir: t.Type<string, string, unknown>;
    verbose: t.Type<Function, Function, unknown>;
}>;
export declare const ModelOptions: t.PartialC<{
    index: t.UnionC<[t.StringC, t.ArrayC<t.UnionC<[t.StringC, t.ArrayC<t.StringC>]>>, t.RecordC<t.StringC, t.UnionC<[t.StringC, t.ArrayC<t.StringC>]>>]>;
    primary: t.ArrayC<t.StringC>;
    unique: t.ArrayC<t.StringC>;
    timestamps: t.BooleanC;
}>;
export declare const AggregateOptions: t.PartialC<{
    distinct: t.BooleanC;
    group: t.UnionC<[t.StringC, t.ArrayC<t.StringC>]>;
    order: t.UnionC<[t.StringC, t.TupleC<[t.StringC, t.StringC]>]>;
}>;
export declare const CreateOptions: t.PartialC<{
    raw: t.BooleanC;
}>;
export declare const FindOptions: t.PartialC<{
    limit: t.NumberC;
    order: t.UnionC<[t.StringC, t.TupleC<[t.StringC, t.StringC]>]>;
    random: t.BooleanC;
    raw: t.BooleanC;
    skip: t.NumberC;
}>;
export declare const UpdateOptions: t.PartialC<{
    raw: t.BooleanC;
}>;
export declare const ColumnKind: t.RefinementC<t.UnionC<[t.StringC, t.FunctionC]>>;
export declare const ColumnDescriptor: t.PartialC<{
    defaultTo: t.AnyC;
    index: t.StringC;
    notNullable: t.BooleanC;
    nullable: t.BooleanC;
    primary: t.BooleanC;
    unique: t.BooleanC;
    type: t.RefinementC<t.UnionC<[t.StringC, t.FunctionC]>>;
    get: t.FunctionC;
    set: t.FunctionC;
}>;
export declare const WhereTuple: t.UnionC<[t.TupleC<[t.StringC, t.AnyC]>, t.TupleC<[t.StringC, t.StringC, t.AnyC]>]>;
export declare const WhereClause: t.UnionC<[t.UnionC<[t.TupleC<[t.StringC, t.AnyC]>, t.TupleC<[t.StringC, t.StringC, t.AnyC]>]>, t.UnknownRecordC]>;
export declare const WhereMultiple: t.ArrayC<t.UnionC<[t.UnionC<[t.TupleC<[t.StringC, t.AnyC]>, t.TupleC<[t.StringC, t.StringC, t.AnyC]>]>, t.UnknownRecordC]>>;
export declare type Index = t.TypeOf<typeof Index>;
export declare type Order = t.TypeOf<typeof OrderClause>;
export declare type Group = t.TypeOf<typeof GroupClause>;
export declare type TrilogyParams = [string, TrilogyOptions?];
export declare type TrilogyOptions = t.TypeOf<typeof TrilogyOptions>;
export declare type ModelOptions = t.TypeOf<typeof ModelOptions>;
export declare type AggregateOptions = t.TypeOf<typeof AggregateOptions>;
export declare type CreateOptions = t.TypeOf<typeof CreateOptions>;
export declare type UpdateOptions = t.TypeOf<typeof UpdateOptions>;
export declare type FindOptions = t.TypeOf<typeof FindOptions>;
export declare type ColumnKind = t.TypeOf<typeof ColumnKind>;
export declare type ColumnDescriptor = t.TypeOf<typeof ColumnDescriptor>;
export declare type SchemaRaw<D = LooseObject> = {
    [P in keyof Partial<D>]: ColumnKind | ColumnDescriptor;
};
export declare type Schema<D = LooseObject> = {
    [P in keyof Partial<D>]: ColumnDescriptor;
};
export declare type SqlJsResponse = Array<{
    columns: string[];
    values: any[];
}>;
export declare type WhereTuple = t.TypeOf<typeof WhereTuple>;
export declare type WhereClause = t.TypeOf<typeof WhereClause>;
export declare type WhereMultiple = WhereClause[];
export declare type StorageType = string | number | Date | null | undefined;
export declare type ReturnType = string | number | boolean | null | undefined | any[] | Date | LooseObject;
export declare type ReturnDict = Record<string, ReturnType>;
export declare type CastToDefinition = Record<string, StorageType> | [string, StorageType] | [string, string, StorageType] | WhereMultiple | never;
