"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const t = require("io-ts");
const ThrowReporter_1 = require("io-ts/lib/ThrowReporter");
const constants_1 = require("./constants");
const util_1 = require("./util");
exports.raise = ThrowReporter_1.ThrowReporter.report;
function validate(value, type, defaultValue = {}) {
    const result = type.decode(value);
    exports.raise(result);
    return result.getOrElse(defaultValue);
}
exports.validate = validate;
// based on `withDefault` from io-ts tests: https://git.io/vNGS6
function withDefault(type, defaultValue) {
    const value = util_1.isFunction(defaultValue)
        ? defaultValue()
        : defaultValue;
    return new t.Type(`withDefault(${type.name}, ${JSON.stringify(value)})`, type.is, (v, c) => type.validate(v != null ? v : value, c), type.encode);
}
exports.withDefault = withDefault;
exports.Index = t.union([
    t.string,
    t.array(t.union([t.string, t.array(t.string)])),
    t.dictionary(t.string, t.union([t.string, t.array(t.string)]))
]);
exports.GroupClause = t.union([
    t.string,
    t.array(t.string)
]);
exports.OrderClause = t.union([
    t.string,
    t.tuple([t.string, t.string])
]);
exports.TrilogyOptions = t.partial({
    client: withDefault(t.union([t.literal('sqlite3'), t.literal('sql.js')]), 'sqlite3'),
    connection: withDefault(t.partial({
        filename: t.string
    }), {}),
    dir: withDefault(t.string, process.cwd),
    verbose: withDefault(t.Function, () => t.identity)
});
exports.ModelOptions = t.partial({
    index: exports.Index,
    primary: t.array(t.string),
    unique: t.array(t.string),
    timestamps: t.boolean
});
exports.AggregateOptions = t.partial({
    distinct: t.boolean,
    group: exports.GroupClause,
    order: exports.OrderClause
});
exports.CreateOptions = t.partial({
    raw: t.boolean
});
exports.FindOptions = t.partial({
    limit: t.number,
    order: exports.OrderClause,
    random: t.boolean,
    raw: t.boolean,
    skip: t.number
});
exports.UpdateOptions = t.partial({
    raw: t.boolean
});
exports.ColumnKind = t.refinement(t.union([t.string, t.Function]), value => {
    const type = util_1.isFunction(value) ? value.name : String(value);
    return constants_1.COLUMN_TYPES.includes(type.toLowerCase());
});
exports.ColumnDescriptor = t.partial({
    defaultTo: t.any,
    index: t.string,
    notNullable: t.boolean,
    nullable: t.boolean,
    primary: t.boolean,
    unique: t.boolean,
    type: exports.ColumnKind,
    get: t.Function,
    set: t.Function
});
exports.WhereTuple = t.union([
    t.tuple([t.string, t.any]),
    t.tuple([t.string, t.string, t.any])
]);
exports.WhereClause = t.union([
    exports.WhereTuple,
    t.Dictionary
]);
exports.WhereMultiple = t.array(exports.WhereClause);
