"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
function eachObj(collection, fn) {
    const keys = Object.keys(collection);
    for (let i = 0; i < keys.length; i++) {
        const key = keys[i];
        const value = collection[key];
        if (fn(value, key) === false)
            break;
    }
}
exports.eachObj = eachObj;
function mapObj(collection, fn) {
    const result = {};
    eachObj(collection, (value, key) => {
        result[key] = fn(value, key);
    });
    return result;
}
exports.mapObj = mapObj;
exports.isObject = (value) => (value && value.constructor === Object) || false;
exports.isFunction = (value) => typeof value === 'function';
exports.isString = (value) => typeof value === 'string';
exports.isNumber = (value) => typeof value === 'number';
exports.isNil = (value) => value == null;
exports.toArray = (value) => Array.isArray(value) ? value : exports.isNil(value) ? [] : [value];
exports.defaultTo = (value, fallback) => exports.isNil(value) ? fallback : value;
function invariant(condition, message) {
    if (!condition) {
        throw new Error(message || 'Invariant Violation');
    }
    return condition;
}
exports.invariant = invariant;
function makeDirPath(path) {
    const mode = parseInt('0777', 8);
    try {
        fs_1.mkdirSync(path, mode);
        return true;
    }
    catch (err) {
        if (err.code === 'EEXIST') {
            return fs_1.statSync(path).isDirectory();
        }
        if (err.code === 'ENOENT') {
            const target = path_1.dirname(path);
            return (target !== path &&
                makeDirPath(target) &&
                (Boolean(fs_1.mkdirSync(path, mode)) || true));
        }
        return false;
    }
}
exports.makeDirPath = makeDirPath;
