import * as t from 'io-ts';
import { ThrowReporter } from 'io-ts/lib/ThrowReporter';
import { COLUMN_TYPES } from './constants';
import { isFunction } from './util';
export const raise = ThrowReporter.report;
export function validate(value, type, defaultValue = {}) {
    const result = type.decode(value);
    raise(result);
    return result.getOrElse(defaultValue);
}
// based on `withDefault` from io-ts tests: https://git.io/vNGS6
export function withDefault(type, defaultValue) {
    const value = isFunction(defaultValue)
        ? defaultValue()
        : defaultValue;
    return new t.Type(`withDefault(${type.name}, ${JSON.stringify(value)})`, type.is, (v, c) => type.validate(v != null ? v : value, c), type.encode);
}
export const Index = t.union([
    t.string,
    t.array(t.union([t.string, t.array(t.string)])),
    t.dictionary(t.string, t.union([t.string, t.array(t.string)]))
]);
export const GroupClause = t.union([
    t.string,
    t.array(t.string)
]);
export const OrderClause = t.union([
    t.string,
    t.tuple([t.string, t.string])
]);
export const TrilogyOptions = t.partial({
    client: withDefault(t.union([t.literal('sqlite3'), t.literal('sql.js')]), 'sqlite3'),
    connection: withDefault(t.partial({
        filename: t.string
    }), {}),
    dir: withDefault(t.string, process.cwd),
    verbose: withDefault(t.Function, () => t.identity)
});
export const ModelOptions = t.partial({
    index: Index,
    primary: t.array(t.string),
    unique: t.array(t.string),
    timestamps: t.boolean
});
export const AggregateOptions = t.partial({
    distinct: t.boolean,
    group: GroupClause,
    order: OrderClause
});
export const CreateOptions = t.partial({
    raw: t.boolean
});
export const FindOptions = t.partial({
    limit: t.number,
    order: OrderClause,
    random: t.boolean,
    raw: t.boolean,
    skip: t.number
});
export const UpdateOptions = t.partial({
    raw: t.boolean
});
export const ColumnKind = t.refinement(t.union([t.string, t.Function]), value => {
    const type = isFunction(value) ? value.name : String(value);
    return COLUMN_TYPES.includes(type.toLowerCase());
});
export const ColumnDescriptor = t.partial({
    defaultTo: t.any,
    index: t.string,
    notNullable: t.boolean,
    nullable: t.boolean,
    primary: t.boolean,
    unique: t.boolean,
    type: ColumnKind,
    get: t.Function,
    set: t.Function
});
export const WhereTuple = t.union([
    t.tuple([t.string, t.any]),
    t.tuple([t.string, t.string, t.any])
]);
export const WhereClause = t.union([
    WhereTuple,
    t.Dictionary
]);
export const WhereMultiple = t.array(WhereClause);
